import cachetools
import asyncache
import httpx
from erniePySDK.Errors import ApiKeyOrSecretKeyError


# accessToken 获取周期
# 有效时间是 30 天，提前一天获取
TTL = 29 * 24 * 60 * 60


@cachetools.cached(cache=cachetools.TTLCache(maxsize=1, ttl=TTL))
def getAccessToken(apiKey: str, secretKey: str) -> str:
    url = "https://aip.baidubce.com/oauth/2.0/token"
    params = {
        "grant_type": "client_credentials",
        "client_id": apiKey,
        "client_secret": secretKey,
    }
    with httpx.Client() as client:
        resp = client.post(url=url, params=params)
        try:
            return resp.json()["access_token"]
        except:
            raise ApiKeyOrSecretKeyError("apiKey or secretKey error")


@asyncache.cached(cache=cachetools.TTLCache(maxsize=1, ttl=TTL))
async def asyncGetAccessToken(apiKey: str, secretKey: str) -> str:
    url = "https://aip.baidubce.com/oauth/2.0/token"
    params = {
        "grant_type": "client_credentials",
        "client_id": apiKey,
        "client_secret": secretKey,
    }
    async with httpx.AsyncClient() as client:
        resp = await client.post(url=url, params=params)
        try:
            return resp.json()["access_token"]
        except:
            raise ApiKeyOrSecretKeyError("apiKey or secretKey error")



from ast import Dict
from typing import List
from erniePySDK import getAccessToken, asyncGetAccessToken
import json
import httpx


class EmbeddingV1:
    def __init__(
        self,
        apiKey: str,
        secretKey: str,
    ):
        self.apiKey = apiKey
        self.secretKey = secretKey
        self.modelUrl = "https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/embeddings/embedding-v1"

    def embedding(self, texts: List[str]) -> Dict:
        """
        输入文本以获取embeddings。说明：
            （1）文本数量不超过16
            （2）每个文本长度不超过 384个token
        """

        url = (
            self.modelUrl
            + "?access_token="
            + getAccessToken(apiKey=self.apiKey, secretKey=self.secretKey)
        )
        payload = json.dumps({"input": texts})
        headers = {"Content-Type": "application/json"}
        with httpx.Client() as client:
            resp = client.post(url=url, data=payload, headers=headers, timeout=60) # type: ignore
        return resp.json()

    async def asyncEmbedding(
        self,
        texts: List[str],
    ) -> Dict:
        access_token = await asyncGetAccessToken(
            apiKey=self.apiKey, secretKey=self.secretKey
        )
        url = self.modelUrl + "?access_token=" + access_token
        payload = json.dumps({"input": texts})
        headers = {"Content-Type": "application/json"}
        async with httpx.AsyncClient() as client:
            resp = await client.post(url=url, data=payload, headers=headers, timeout=60) # type: ignore
        return resp.json()



from erniePySDK import getAccessToken, asyncGetAccessToken
import httpx
from typing import Any, AsyncIterator, List, Dict, Iterator
import json


class Bloomz7B:
    def __init__(
        self,
        apiKey: str,
        secretKey: str,
    ):
        self.apiKey = apiKey
        self.secretKey = secretKey
        self.modelUrl = "https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/bloomz_7b1"

    def chat(
        self,
        messages: List[Dict[str, str]],
        stream: bool = False,
    ) -> Iterator[Dict[str, Any]]:
        access_token = getAccessToken(apiKey=self.apiKey, secretKey=self.secretKey)
        url = self.modelUrl + "?access_token=" + access_token

        payload = json.dumps(
            {
                "messages": messages,
                "stream": stream,
            }
        )

        headers = {"Content-Type": "application/json"}

        if stream:
            with httpx.Client() as client:
                with client.stream(
                    method="POST", url=url, data=payload, headers=headers  # type: ignore
                ) as resp:
                    for line in resp.iter_lines():
                        if line:
                            data = line.split("data: ")[1]
                            yield json.loads(data)

        else:
            with httpx.Client() as client:
                resp = client.post(url=url, data=payload, headers=headers, timeout=60)  # type: ignore
                yield resp.json()

    async def asyncChat(
        self,
        messages: List[Dict[str, str]],
        stream: bool = False,
    ) -> AsyncIterator[Dict[str, Any]]:
        access_token = await asyncGetAccessToken(
            apiKey=self.apiKey, secretKey=self.secretKey
        )
        url = self.modelUrl + "?access_token=" + access_token

        payload = json.dumps(
            {
                "messages": messages,
                "stream": stream,
            }
        )

        headers = {"Content-Type": "application/json"}

        if stream:
            async with httpx.AsyncClient() as client:
                async with client.stream(
                    method="POST", url=url, data=payload, headers=headers # type: ignore
                ) as resp:
                    async for line in resp.aiter_lines():
                        if line:
                            data = line.split("data: ")[1]
                            yield json.loads(data)
        else:
            async with httpx.AsyncClient() as client:
                resp = await client.post(
                    url=url, data=payload, headers=headers, timeout=60 # type: ignore
                )
                yield resp.json()



from erniePySDK.Client import getAccessToken, asyncGetAccessToken
from erniePySDK.ErnieBot import ErnieBot
from erniePySDK.ErnieBotTurbo import ErnieBotTurbo
from erniePySDK.Bloomz7B import Bloomz7B
from erniePySDK.EmbeddingV1 import EmbeddingV1


__version__ = "0.1.2"



from erniePySDK import ErnieBot


class ErnieBotTurbo(ErnieBot):
    def __init__(
        self,
        apiKey: str,
        secretKey: str,
    ):
        super().__init__(apiKey, secretKey)
        self.modelUrl = "https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/eb-instant"
